﻿using UnityEngine;
using System.Collections;

public class ConnectedObjects : MonoBehaviour {

    public Camera parentCamera;
    public ConnectedObjects otherObject;
    public bool isCurrentObject;
    private ConnectedObjects currentObject;
    public Vector2 worldRelationPosition;
    private const float objectSwitchMargin = 0.0f;
    protected Rigidbody2D rb;

    void Start() {
        StartAdditions();
        rb = GetComponent<Rigidbody2D>();
        if ( isCurrentObject ) {
            takeCurrentObjectRole();
        }
    }

    protected virtual void StartAdditions() {

    }
	
	void Update() {
        updateAlways();
        if ( isCurrentObject ) {
            updateWhenCurrentObject();
        } else {
            updateWhenNotCurrentObject();
        }
	}

    void LateUpdate() {
        updateCurrentPlayer();
    }

    protected virtual void updateCurrentPlayer() {
        if ( !isCurrentObject ) {
            if ( transform.position.y > parentCamera.transform.position.y - parentCamera.orthographicSize + objectSwitchMargin &&
                 transform.position.y < parentCamera.transform.position.y + parentCamera.orthographicSize - objectSwitchMargin ) {
                takeCurrentObjectRole();
            }
        }
    }

    protected virtual void takeCurrentObjectRole() {
        isCurrentObject = true;
        currentObject = this;
        otherObject.setCurrentObject( this );
    }

    protected virtual void updateAlways() {
    }

    protected virtual void updateWhenCurrentObject() {
    }

    protected virtual void updateWhenNotCurrentObject() {
        rb.velocity = currentObject.getVelocity();
        transform.position = getPositionFromCurrentObject();
        transform.rotation = currentObject.transform.rotation;
    }

    Vector3 getPositionFromCurrentObject() {
        return new Vector3(
                parentCamera.transform.position.x
                    + currentObject.getRelativePosition().x
                    - ( ( worldRelationPosition.x - currentObject.worldRelationPosition.x ) * parentCamera.orthographicSize * 2 ),
                parentCamera.transform.position.y
                    + currentObject.getRelativePosition().y
                    - ( ( worldRelationPosition.y - currentObject.worldRelationPosition.y ) * parentCamera.orthographicSize * 2 ),
                transform.position.z );
    }

    public void setCurrentObject( ConnectedObjects obj ) {
        this.isCurrentObject = false;
        this.currentObject = obj;
    }

    public Vector2 getRelativePosition() {
        Vector2 relativePosition = new Vector2( transform.position.x - parentCamera.transform.position.x,
                                                transform.position.y - parentCamera.transform.position.y );
        return relativePosition;
    }

    public Vector2 getVelocity() {
        return rb.velocity;
    }

    public bool getIsCurrentPlayer() {
        return isCurrentObject;
    }
}
