﻿using UnityEngine;
using System.Collections;

public class WorldController : MonoBehaviour {
    public Color backgroundStartingColor;
    private Color backgroundWinningColor;
    public Color backgroundLosingColor;
    public Color foregroundStartingColor;
    public Color foregroundLosingColor;
    private GlobalController globalController;
    private Camera camera;

    void Start() {
        camera = GetComponent<Camera>();
        backgroundWinningColor = camera.backgroundColor;
        globalController = GameObject.FindObjectOfType<GlobalController>();
        if ( !globalController ) {
            Debug.LogError( "No global world controller found. Please add one." );
        }
    }

    void Update() {
        camera.backgroundColor = getFadedColor( backgroundStartingColor, getTargetColor() );
    }

    Color getTargetColor() {
        if ( isWinning() ) {
            return backgroundWinningColor;
        }
        return backgroundLosingColor;
    }

    public bool isWinning() {
        return globalController.isWinning();
    }

    public Color getFadedColor( Color startingColor, Color targetColor ) {
        Color colorDifference = ( targetColor - startingColor );
        return startingColor + colorDifference * globalController.getProgress();
    }
}