﻿using UnityEngine;
using System.Collections.Generic;
using System.Collections;

public class Player : MonoBehaviour {

    public Camera parentCamera;
    private List<Player> otherPlayers;
    public bool isCurrentPlayer;
    private Player currentPlayer;
    public Vector2 worldRelationPosition;
    private const float playerSwitchMargin = 0.0f;
    protected Rigidbody2D rb;
    private GlobalController globalController;

    void Start() {
        globalController = GameObject.FindObjectOfType<GlobalController>();
        otherPlayers = new List<Player>();
        Player[] playerList = GameObject.FindObjectsOfType<Player>();
        rb = GetComponent<Rigidbody2D>();
        foreach ( Player player in playerList ) {
            if ( player != this ) {
                otherPlayers.Add( player );
            }
        }
        if ( isCurrentPlayer ) {
            takeCurrentPlayerRole();
        }
        StartAdditions();
    }

    protected virtual void StartAdditions() {

    }
	
	void Update() {
        if ( globalController.gameIsOn ) {
            if ( isCurrentPlayer ) {
                updateWhenCurrentPlayer();
            } else {
                updateWhenNotCurrentPlayer();
            }
            updateAlways();
        } else {
            gameObject.SetActive( false );
        }
	}

    void LateUpdate() {
        updateCurrentPlayer();
    }

    void updateCurrentPlayer() {
        if ( !isCurrentPlayer ) {
            if ( transform.position.y > parentCamera.transform.position.y - parentCamera.orthographicSize + playerSwitchMargin &&
                 transform.position.y < parentCamera.transform.position.y + parentCamera.orthographicSize - playerSwitchMargin ) {
                takeCurrentPlayerRole();
            }
        }
    }

    protected virtual void takeCurrentPlayerRole() {
        isCurrentPlayer = true;
        currentPlayer = this;
        foreach ( Player player in otherPlayers ) {
            player.setCurrentPlayer( this );
        }
    }

    protected virtual void updateAlways() {
    }

    protected virtual void updateWhenCurrentPlayer() {
        float speedMultiplier = 10.0f;
        rb.velocity = new Vector2( Input.GetAxis( "Horizontal" ) * speedMultiplier, Input.GetAxis( "Vertical" ) * speedMultiplier );
    }

    protected virtual void updateWhenNotCurrentPlayer() {
        rb.velocity = currentPlayer.getVelocity();
        transform.position = getPositionFromCurrentPlayer();
        transform.rotation = currentPlayer.transform.rotation;
    }

    Vector3 getPositionFromCurrentPlayer() {
        return new Vector3(
                parentCamera.transform.position.x
                    + currentPlayer.getRelativePosition().x
                    - ( ( worldRelationPosition.x - currentPlayer.worldRelationPosition.x ) * parentCamera.orthographicSize * 2 ),
                parentCamera.transform.position.y
                    + currentPlayer.getRelativePosition().y
                    - ( ( worldRelationPosition.y - currentPlayer.worldRelationPosition.y ) * parentCamera.orthographicSize * 2 ),
                transform.position.z );
    }

    public void setCurrentPlayer( Player player ) {
        this.isCurrentPlayer = false;
        this.currentPlayer = player;
    }

    public Vector2 getRelativePosition() {
        Vector2 relativePosition = new Vector2( transform.position.x - parentCamera.transform.position.x,
                                                transform.position.y - parentCamera.transform.position.y );
        return relativePosition;
    }

    public Vector2 getVelocity() {
        return rb.velocity;
    }

    public bool getIsCurrentPlayer() {
        return isCurrentPlayer;
    }
}
