﻿using UnityEngine;
using System.Collections;

public class PlayerBlackWorld : Player {

    public GameObject tongue;
    private bool tongueIsReaching = false;
    private bool tongueHasReachedTarget = false;
    private float tongueSpeed = 0.5f;
    private Vector2 tongueTarget;
    private Vector2 tongueHome;
    public GameObject tongueHomeObject;
    private LineRenderer tongueLineRenderer;
    private float tongueReach = 3.0f;

    protected override void StartAdditions() {
        base.StartAdditions();
        tongueLineRenderer = tongue.GetComponent<LineRenderer>();
    }

    protected override void updateWhenCurrentPlayer() {
        handleMovement();
        handleFireInput();
        if ( rb.velocity.magnitude > 0 ) {
            float angle = Mathf.Atan2( rb.velocity.y, rb.velocity.x ) * Mathf.Rad2Deg;
            transform.rotation = Quaternion.AngleAxis( angle, Vector3.forward );
        }
    }

    void handleMovement() {
        float speedMultiplier = 10.0f;
        rb.velocity = new Vector2( Input.GetAxis( "Horizontal" ) * speedMultiplier, Input.GetAxis( "Vertical" ) * speedMultiplier );
    }

    void handleFireInput() {
        setTongueTargetIfFiring();
    }

    protected override void updateAlways() {
        base.updateAlways();
        updateTonguePositionAndStatus();
        updateTongueLine();
        tongueHome = new Vector2( tongueHomeObject.transform.position.x, tongueHomeObject.transform.position.y );
    }

    void setTongueTargetIfFiring() {
        if ( !tongueIsReaching ) {
            if ( Input.GetAxis( "Fire1" ) > 0 ) {
                Vector3 mouseWorldPosition = parentCamera.ScreenToWorldPoint( Input.mousePosition );
                Vector2 playerPosition = new Vector2( tongueHome.x, tongueHome.y );
                tongueTarget = new Vector2( mouseWorldPosition.x, mouseWorldPosition.y );
                Vector2 positionDifference = tongueTarget - playerPosition;
                if ( positionDifference.magnitude > tongueReach ) {
                    tongueTarget = playerPosition + positionDifference.normalized * tongueReach;
                }
                tongueIsReaching = true;
            }
        }
    }

    void updateTonguePositionAndStatus() {
        if ( tongueIsReaching ) {
            Vector2 tonguePosition = new Vector2( tongue.transform.position.x, tongue.transform.position.y );
            Vector2 targetPosition;
            if ( !tongueHasReachedTarget ) {
                Vector2 positionDifference = tongueTarget - tonguePosition;
                if ( positionDifference.magnitude > tongueSpeed ) {
                    targetPosition = tonguePosition + positionDifference.normalized * tongueSpeed;
                } else {
                    targetPosition = tongueTarget;
                    tongueHasReachedTarget = true;
                }
            } else {
                Vector2 returnTarget = new Vector2( tongueHome.x, tongueHome.y );
                Vector2 positionDifference = returnTarget - tonguePosition;
                if ( positionDifference.magnitude > tongueSpeed ) {
                    targetPosition = tonguePosition + positionDifference.normalized * tongueSpeed;
                } else {
                    targetPosition = returnTarget;
                    tongueHasReachedTarget = false;
                    tongueIsReaching = false;
                }
            }
            tongue.transform.position = new Vector3( targetPosition.x, targetPosition.y, tongue.transform.position.z );
        }
    }

    void updateTongueLine() {
        tongueLineRenderer.SetPosition( 0, tongue.transform.position );
        tongueLineRenderer.SetPosition( 1, tongueHomeObject.transform.position );
    }

    protected override void updateWhenNotCurrentPlayer() {
        base.updateWhenNotCurrentPlayer();
    }

    public bool getTongueIsReaching() {
        return tongueIsReaching;
    }
}
