﻿using UnityEngine;
using System.Collections;

public class PlayerWhiteWorld : Player {

    private float jetpackPower = 20.0f;
    private float sideSpeed = 7.5f;
    public ParticleSystem jetParticleSystem;
    public ParticleSystem pushParticleSystem;
    private int disabledTimer = 0;
    public int attackCooldown = 40;
    public float attackReach = 5.0f;
    public float attackForce = 10.0f;

    void FixedUpdate() {
        disabledTimer = ( int )Mathf.Max( 0, --disabledTimer );
    }

    protected override void updateWhenCurrentPlayer() {
        handleFireInput();
        handleJetpackInput();
        transform.rotation = Quaternion.AngleAxis( 90.0f, Vector3.forward );
    }

    protected override void updateWhenNotCurrentPlayer() {
        base.updateWhenNotCurrentPlayer();
    }

    protected override void updateAlways() {
        base.updateAlways();
        if ( Input.GetAxis( "Horizontal" ) > 0 ) {
            transform.localScale = new Vector3( transform.localScale.x, 1.0f, transform.localScale.z );
        } else if ( Input.GetAxis( "Horizontal" ) < 0 ) {
            transform.localScale = new Vector3( transform.localScale.x, -1.0f, transform.localScale.z );
        }
    }

    void disable() {
        disabledTimer = attackCooldown;
    }

    bool isEnabled() {
        return disabledTimer <= 0;
    }

    protected override void takeCurrentPlayerRole() {
        base.takeCurrentPlayerRole();
        rb.velocity = new Vector2( 0.0f, 1.0f );
    }

    void handleFireInput() {
        if ( isEnabled() ) {
            if ( Input.GetAxis( "Fire1" ) > 0 ) {
                disable();
                pushOtherRigidbodies();
            }
        }
    }

    void handleJetpackInput() {
        addJetpackForce();
        addSidewaysForce();
    }

    void pushOtherRigidbodies() {
        Collider2D[] otherBodies = Physics2D.OverlapCircleAll( transform.position, attackReach );
        Vector2 position = new Vector2( transform.position.x, transform.position.y );
        foreach ( Collider2D collider in otherBodies ) {
            Rigidbody2D otherBody = collider.GetComponent<Rigidbody2D>();
            if ( otherBody && otherBody != rb ) {
                Vector2 otherPosition = new Vector2( otherBody.transform.position.x, otherBody.transform.position.y );
                Vector2 positionDifference = otherPosition - position;
                Vector2 direction = positionDifference.normalized;
                Vector2 force = direction * attackForce;
                otherBody.AddForce( force, ForceMode2D.Impulse );
            }
        }
        pushParticleSystem.Play();
    }

    void addJetpackForce() {
        if ( Input.GetAxis( "Vertical" ) > 0 && isEnabled() ) {
            rb.AddForce( new Vector2( 0.0f, jetpackPower ) );
            jetParticleSystem.Play();
        } else {
            jetParticleSystem.Stop();
        }
    }

    void addSidewaysForce() {
        if ( isEnabled() ) {
            rb.AddForce( new Vector2( Input.GetAxis( "Horizontal" ) * sideSpeed, 0.0f ) );
        }
    }
}
